@@ -3,7 +3,7 @@
 
 #define CMOR_VERSION_MAJOR 3
 #define CMOR_VERSION_MINOR 2
-#define CMOR_VERSION_PATCH 1
+#define CMOR_VERSION_PATCH 5
 
 #define CMOR_CF_VERSION_MAJOR 1
 #define CMOR_CF_VERSION_MINOR 6
@@ -11,13 +11,14 @@
 #define CMOR_MAX_STRING 1024
 #define CMOR_DEF_ATT_STR_LEN 256
 #define CMOR_MAX_ELEMENTS 500
+#define CMOR_MAX_FORMULA 40
 #define CMOR_MAX_AXES CMOR_MAX_ELEMENTS*3
 #define CMOR_MAX_VARIABLES CMOR_MAX_ELEMENTS
 #define CMOR_MAX_GRIDS 100
 #define CMOR_MAX_DIMENSIONS 7
 #define CMOR_MAX_ATTRIBUTES 100
 #define CMOR_MAX_ERRORS 10
-#define CMOR_MAX_TABLES 10
+#define CMOR_MAX_TABLES 30
 #define CMOR_MAX_GRID_ATTRIBUTES 25
 #define CMOR_MAX_JSON_ARRAY 50
 #define CMOR_MAX_JSON_OBJECT 250
@@ -71,7 +72,7 @@
 #define AXIS_ATT_AXIS            "axis"
 #define AXIS_ATT_INDEXONLY       "index_only"
 #define AXIS_ATT_MUSTBOUNDS      "must_have_bounds"
-#define AXIS_ATT_MUSTCALLGRID    "must_call_cmmore_grid"
+#define AXIS_ATT_MUSTCALLGRID    "must_call_cmor_grid"
 #define AXIS_ATT_TYPE            "type"
 #define AXIS_ATT_VALIDMIN        "valid_min"
 #define AXIS_ATT_VALIDMAX        "valid_max"
@@ -83,6 +84,7 @@
 #define AXIS_ATT_BOUNDSREQUESTED "bounds_requested"
 #define AXIS_ATT_REQUESTEDBOUNDS "requested_bounds"
 #define AXIS_ATT_REQUESTED       "requested"
+#define AXIS_FORECAST_TIME       "reftime"
 
 #define TABLE_CONTROL_FILENAME    "CMIP6_CV.json"
 #define TABLE_FOUND               -1
@@ -99,7 +101,7 @@
 #define AXIS_ENTRY_FILENAME           "CMIP6_coordinate.json"
 #define CMOR_FORMULA_VAR_FILE         GLOBAL_INTERNAL"FORMULA_VAR_FILE"
 #define VARIABLE_ATT_UNITS            "units"
-#define VARIABLE_ATT_MISSINGVALUES    "missing_values"
+#define VARIABLE_ATT_MISSINGVALUES    "missing_value"
 #define VARIABLE_ATT_FILLVAL          "_FillValue"
 #define VARIABLE_ATT_STANDARDNAME     "standard_name"
 #define VARIABLE_ATT_LONGNAME         "long_name"
@@ -125,6 +127,7 @@
 #define VARIABLE_ATT_VALIDMAX         "valid_max"
 #define VARIABLE_ATT_MINMEANABS       "ok_min_mean_abs"
 #define VARIABLE_ATT_MAXMEANABS       "ok_max_mean_abs"
+#define VARIABLE_ATT_CHUNKING         "chunk_dimensions"
 #define VARIABLE_ATT_SHUFFLE          "shuffle"
 #define VARIABLE_ATT_DEFLATE          "deflate"
 #define VARIABLE_ATT_DEFLATELEVEL     "deflate_level"
@@ -133,7 +136,7 @@
 #define VARIABLE_ATT_FLAGVALUES       "flag_values"
 #define VARIABLE_ATT_FLAGMEANINGS     "flag_meanings"
 #define VARIABLE_ATT_OUTNAME          "out_name"
-
+#define COMMENT_VARIABLE_ZFACTOR      "use formula table"
 #define GLOBAL_SEPARATORS             "><"
 #define GLOBAL_OPENOPTIONAL           "["
 #define GLOBAL_CLOSEOPTIONAL          "]"
@@ -153,6 +156,8 @@
 #define GLOBAL_ATT_VARIABLE_ID        "variable_id"
 #define GLOBAL_ATT_SOURCE_ID          "source_id"
 #define GLOBAL_ATT_SOURCE             "source"
+#define GLOBAL_ATT_SUB_EXPT_ID        "sub_experiment_id"
+#define GLOBAL_ATT_SUB_EXPT           "sub_experiment"
 #define GLOBAL_ATT_SOURCE_TYPE        "source_type"
 #define GLOBAL_ATT_CONVENTIONS        "Conventions"
 #define GLOBAL_ATT_CREATION_DATE      "creation_date"
@@ -166,6 +171,7 @@
 #define GLOBAL_ATT_REALM              "realm"
 #define GLOBAL_ATT_TRACKING_ID        "tracking_id"
 #define GLOBAL_ATT_VARIANT_LABEL      "variant_label"
+#define GLOBAL_ATT_MEMBER_ID          GLOBAL_INTERNAL"member_id"
 #define GLOBAL_ATT_DATASPECSVERSION   "data_specs_version"
 #define GLOBAL_ATT_FREQUENCY          "frequency"
 #define GLOBAL_ATT_LICENSE            "license"
@@ -183,31 +189,42 @@
 #define GLOBAL_CV_FILENAME            GLOBAL_INTERNAL"control_vocabulary_file"
 #define GLOBAL_IS_CMIP6               GLOBAL_INTERNAL"cmip6_option"
 
+#define NO_PARENT                     "no parent"
+#define NONE                          "none"
+#define BRANCH_METHOD                 "branch_method"
+#define BRANCH_TIME_IN_CHILD          "branch_time_in_child"
+#define BRANCH_TIME_IN_PARENT         "branch_time_in_parent"
+#define PARENT_ACTIVITY_ID            "parent_activity_id"
+#define PARENT_EXPERIMENT_ID          "parent_experiment_id"
+#define PARENT_MIP_ERA                "parent_mip_era"
+#define PARENT_SOURCE_ID              "parent_source_id"
+#define PARENT_TIME_UNITS             "parent_time_units"
+#define PARENT_VARIANT_LABEL          "parent_variant_label"
 
 #define JSON_KEY_HEADER               "Header"
 #define JSON_KEY_EXPERIMENT           "experiments"
 #define JSON_KEY_AXIS_ENTRY           "axis_entry"
 #define JSON_KEY_VARIABLE_ENTRY       "variable_entry"
+#define JSON_KEY_FORMULA_ENTRY        "formula_entry"
 #define JSON_KEY_MAPPING_ENTRY        "mapping_entry"
 #define JSON_KEY_CV_ENTRY             "CV"
 
 #define CV_KEY_REQUIRED_GBL_ATTRS     "required_global_attributes"
-#define CV_KEY_INSTITUTION_IDS        "institution_id"
-#define CV_KEY_EXPERIMENT_IDS         "experiment_id"
+#define CV_KEY_INSTITUTION_ID         "institution_id"
+#define CV_KEY_EXPERIMENT_ID          "experiment_id"
 #define CV_KEY_SOURCE_IDS             "source_id"
 #define CV_KEY_GRID_LABELS            "grid_label"
 #define CV_KEY_GRID_RESOLUTION        "nominal_resolution"
 #define CV_KEY_GRIDLABEL_GR           "gr"
-#define CV_KEY_SOURCE_LABEL           "label"
-
+#define CV_KEY_SOURCE_LABEL           "source"
+#define CV_KEY_SUB_EXPERIMENT_ID      "sub_experiment_id"
 
 #define CV_EXP_ATTR_ADDSOURCETYPE     "additional_allowed_model_components"
 #define CV_EXP_ATTR_REQSOURCETYPE     "required_model_components"
-
+#define CV_EXP_ATTR_DESCRIPTION       "description"
 #define GLOBAL_INT_ATT_PARENT_EXPT    GLOBAL_INTERNAL"parent_experiment"
 #define GLOBAL_ATT_VERSION            GLOBAL_INTERNAL"version"
 
-
 #define TABLE_HEADER_VERSION          "cmor_version"
 #define TABLE_HEADER_GENERIC_LEVS     "generic_levels"
 #define TABLE_HEADER_CF_VERSION       "cf_version"
@@ -216,7 +233,6 @@
 #define TABLE_HEADER_REALM            "realm"
 #define TABLE_HEADER_TABLE_DATE       "table_date"
 #define TABLE_HEADER_FORCINGS         "forcings"
-#define TABLE_HEADER_FREQUENCY        "frequency"
 #define TABLE_HEADER_TABLE_ID         "table_id"
 #define TABLE_HEADER_BASEURL          "baseURL"
 #define TABLE_HEADER_PRODUCT          "product"
@@ -225,6 +241,7 @@
 #define TABLE_HEADER_APRX_INTRVL_ERR  "approx_interval_error"
 #define TABLE_HEADER_APRX_INTRVL_WRN  "approx_interval_warning"
 #define TABLE_HEADER_MISSING_VALUE    "missing_value"
+#define TABLE_HEADER_INT_MISSING_VALUE    "int_missing_value"
 #define TABLE_HEADER_MAGIC_NUMBER     "magic_number"
 #define TABLE_HEADER_DATASPECSVERSION "data_specs_version"
 #define OUTPUT_TEMPLATE_RIPF          "run_variant"
@@ -235,11 +252,14 @@
 #define DIMENSION_ZLEVEL              "zlevel"
 #define DIMENSION_OLEVEL              "olevel"
 
+#define AREA                          "area"
+#define VOLUME                        "volume"
 #define CMIP6                         "CMIP6"
-#define CMOR_DEFAULT_PATH_TEMPLATE    "<mip_era><institute_id><source_id><activity_id><experiment_id><variant_label><table><variable_id><grid_label><version>"
-#define CMOR_DEFAULT_FILE_TEMPLATE    "<variable_id><table><experiment_id><source_id><variant_label><grid_label>"
+#define CMOR_DEFAULT_PATH_TEMPLATE    "<mip_era><activity_id><institution_id><source_id><experiment_id><member_id><table><variable_id><grid_label><version>"
+#define CMOR_DEFAULT_FILE_TEMPLATE    "<variable_id><table><source_id><experiment_id><member_id><grid_label>"
 #define CMOR_DEFAULT_FURTHERURL_TEMPLATE "http://furtherinfo.es-doc.org/<mip_era><institution_id><source_id><experiment_id><sub_experiment_id><variant_label>"
-#define EXTERNAL_VARIABLE_REGEX       "area:[[:blank:]]*([[:alpha:]]+)([[:blank:]]*volume:[[:blank:]]*([[:alpha:]]+))*"
+//#define EXTERNAL_VARIABLE_REGEX       "([[:alpha:]]+):[[:blank:]]*([[:alpha:]]+)[[:blank:]]*([[:alpha:]]+:[[:blank:]]*([[:alpha:]]+))*"
+#define EXTERNAL_VARIABLE_REGEX       "[[:alpha:]]+:[[:blank:]]*([[:alpha:]]+)([[:blank:]]*[[:alpha:]]+:[[:blank:]]*([[:alpha:]]+))*"
 
 extern int USE_NETCDF_4;
 extern int CMOR_MODE;
@@ -248,7 +268,7 @@ extern int CMOR_VERBOSITY;
 extern int CMOR_NETCDF_MODE;
 extern int CV_ERROR;
 
-extern int cleanup_varid;
+extern int clfeanup_varid;
 
 extern int cmor_naxes;
 extern int cmor_nvars;
@@ -388,6 +408,7 @@ typedef struct cmor_variable_def_ {
     float valid_max;
     float ok_min_mean_abs;
     float ok_max_mean_abs;
+    char chunking_dimensions[CMOR_MAX_STRING];
     int shuffle;
     int deflate;
     int deflate_level;
@@ -405,6 +426,7 @@ typedef struct cmor_var_ {
     int ref_table_id;
     int ref_var_id;
     int initialized;
+    int error;
     int closed;
     int nc_var_id;
     int nc_zfactors[CMOR_MAX_VARIABLES];
@@ -436,6 +458,7 @@ typedef struct cmor_var_ {
     float valid_max;
     float ok_min_mean_abs;
     float ok_max_mean_abs;
+    char chunking_dimensions[CMOR_MAX_STRING];
     int shuffle;
     int deflate;
     int deflate_level;
@@ -454,9 +477,11 @@ typedef struct cmor_var_ {
     char current_path[CMOR_MAX_STRING];
     char suffix[CMOR_MAX_STRING];
     int suffix_has_date;
+    char frequency[CMOR_MAX_STRING];
 } cmor_var_t;
 
 extern cmor_var_t cmor_vars[CMOR_MAX_VARIABLES];
+extern cmor_var_t cmor_formula[CMOR_MAX_FORMULA];
 
 typedef struct cmor_mappings_ {
     int nattributes;
@@ -474,6 +499,7 @@ typedef struct {
 typedef struct cmor_table_ {
     int id;
     int nvars;
+    int nformula;
     int naxes;
     int nexps;
     int nmappings;
@@ -488,17 +514,18 @@ typedef struct cmor_table_ {
     char date[CMOR_MAX_STRING];
     cmor_axis_def_t axes[CMOR_MAX_ELEMENTS];
     cmor_var_def_t vars[CMOR_MAX_ELEMENTS];
+    cmor_var_def_t formula[CMOR_MAX_FORMULA];
     cmor_mappings_t mappings[CMOR_MAX_ELEMENTS];
     cmor_CV_def_t *CV;
-    float missing_value;
+    double missing_value;
+    long    int_missing_value;
     double interval;
-    float interval_warning;
-    float interval_error;
+    double interval_warning;
+    double interval_error;
     char URL[CMOR_MAX_STRING];
     char product[CMOR_MAX_STRING];
     char realm[CMOR_MAX_STRING];
     char path[CMOR_MAX_STRING];
-    char frequency[CMOR_MAX_STRING];
     char **forcings;
     int nforcings;
     unsigned char md5[16];
